/*******************************************************************************
*
*
* File:         demo_hal_pwm.c
* Description： pwm demo code
* Version：      V0.1
* Date：         2023-07-27
* Author：       jaya
 *******************************************************************************/

/*******************************************************************************
* 1.Included files
*******************************************************************************/
#include "stdarg.h"
#include "stdlib.h"
#include "stdio.h"
#include "test_cfg_global.h"
#include "hal_pwm.h"
#include "hal_gpio.h"
#include "tau_log.h"
#include "tau_delay.h"
#include "demo_hal_pwm.h"

#if _MODULE_DEMO_PWM_EN
/*******************************************************************************
* 2.Global constant and macro definitions using #define
*******************************************************************************/
#ifdef LOG_TAG
#undef LOG_TAG
#endif
#define LOG_TAG "demo_hal_pwm"

/*******************************************************************************
* 3.Global function and Global variable declarations
*******************************************************************************/

/*******************************************************************************
* 4.Global variable extern declarations
*******************************************************************************/

/*******************************************************************************
* 5.Global function prototypes
*******************************************************************************/
/**
* @brief  数字PWM输出demo
* @param
* @retval bool true/false
*/
bool demo_digital_pwm_output(void)
{
    /* GPIO 初始化,配置PIN15输出PWM, PWM可配置从PIN2/PIN13/PIN15/PIN17/PIN36输出 */
    hal_gpio_set_mode(IO_PIN_15, PIN15_MODE_PWMO);
    //hal_gpio_set_mode(IO_PIN_17, PIN17_MODE_PWMO);

    /* PWM 初始化 ，配置为30K 255阶 */
    if (hal_pwm_init(30000, 255))
    {
        /* PWM enable */
        hal_pwm_enable(true);
        uint8_t i = 0;
        while (i != 255)
        {
            /* PWM 配置占空比, 从1/255开始到100%*/
            hal_pwm_set_duty(i);
            delayMs(20);
            i++;
        }
    }

    /* PWM disable */
    hal_pwm_enable(false);
    /* PWM deinit */
    hal_pwm_deinit();
    TAU_LOGD("demo_digital_pwm_output done\n");
    return true;
}

/**
* @brief  模拟PWM输出demo
* @param
* @retval bool true/false
*/
bool demo_analog_pwm_output_with_vcc(void)
{
    /* 模拟PWM,可选择不从数字IO输出PWM,也可以选择从数字IO输出 */
    hal_gpio_set_mode(IO_PIN_15, PIN15_MODE_PWMO);
    //hal_gpio_set_mode(IO_PIN_17, PIN17_MODE_PWMO);

    /* PWM 初始化 ，配置为30K 255阶 */
    if (hal_pwm_init(30000, 255))
    {
        /* PWM enable */
        hal_pwm_enable(true);
        /* ELVCC 接入3-6V的电源,PWM 配置从ELVCC调试后的电源 */
        hal_pwm_set_elvcc_output(true);

        uint8_t i = 0;
        while (i != 255)
        {
            hal_pwm_set_duty(i);
            delayMs(20);
            i++;
        }
    }

    /* PWM disable */
    hal_pwm_enable(false);
    /* PWM deinit */
    hal_pwm_deinit();
    TAU_LOGD("demo_digital_pwm_output done\n");
    return true;
}

/**
* @brief  pwm demo case
* @param  none
* @retval none
*/
void demo_hal_pwm(void)
{
    TAU_LOGD("pwm dmeo \n");
    //demo_digital_pwm_output();
    demo_analog_pwm_output_with_vcc();
}
#endif
