/*******************************************************************************
*
*
* File:          demo_hal_wdg.c
* Description：  watch dog demo源文件
* Version：      V0.1
* Date：         2023-07-27
* Author：       zzf
 *******************************************************************************/

/*******************************************************************************
* 1.Included files
*******************************************************************************/
#include "hal_wdg.h"
#include "hal_gpio.h"
#include "tau_delay.h"
#include "tau_log.h"

/*******************************************************************************
* 2.Global constant and macro definitions using #define
*******************************************************************************/
#ifdef LOG_TAG
#undef LOG_TAG
#endif
#define LOG_TAG "wdg-log"

#define WDG_CASE_RST            0 //复位模式
#define WDG_CASE_INTR           1 //中断模式

/*******************************************************************************
* 3.Global structures, unions and enumerations using typedef
*******************************************************************************/

/*******************************************************************************
* 4.Global variable extern declarations
*******************************************************************************/
#if WDG_CASE_INTR
static io_pad_e sg_wdg_io_pad = IO_PAD_GPIO1;
static gpio_level_e sg_wdg_io_lvl = IO_LVL_LOW;
/* 循环喂狗 */
static uint8_t sg_feed_cnt = 0;
#endif

/*******************************************************************************
* 5.Global function prototypes
*******************************************************************************/
#if WDG_CASE_RST
/**
* @brief  复位模式测试
* @param  无
* @retval 无
*/
static void demo_wdg_case_rst_mode(void)
{
    TAU_LOGI("reset start...\n");

    /* WDG初始化复位模式 */
    hal_wdg_init();

    /* 启动 */
    hal_wdg_start(WDG_MODE_RESET, 5000);
}
#endif

#if WDG_CASE_INTR
/**
* @brief  测试回调函数
* @param  无
* @retval 无
*/
static void demo_wdg_callback(void *data)
{
    /* 翻转GPIO */
    sg_wdg_io_lvl = (sg_wdg_io_lvl ? IO_LVL_LOW : IO_LVL_HIGH);
    hal_gpio_set_output_data(sg_wdg_io_pad, sg_wdg_io_lvl);
    sg_feed_cnt++;

    if (sg_feed_cnt > 10)
    {
        hal_wdg_stop();
    }
}

/**
* @brief  中断模式测试
* @param  无
* @retval 无
*/
static void demo_wdg_case_int_mode(void)
{
    TAU_LOGI("int start...\n");

    /* WDG初始化 */
    hal_wdg_init();

    /* 设置循环超时 */
    hal_wdg_set_repeat(true);

    /* 设置回调函数 */
    hal_wdg_register_callback(demo_wdg_callback, NULL);

    /*初始化GPIO低状态*/
    hal_gpio_init_output(sg_wdg_io_pad, sg_wdg_io_lvl);

    /* 设置中断模式 */
    hal_wdg_start(WDG_MODE_INTERRUPT, 200);


    while (sg_feed_cnt < 6)
    {
        sg_feed_cnt++;
        hal_wdg_kick_dog();
        delayMs(150);
        TAU_LOGI("feed [%d]th done\n", sg_feed_cnt);
    }

}
#endif

/**
* @brief  demo测试接口函数
* @param  None
* @retval None
*/
void demo_wdg_case(void)
{
    TAU_LOGI("demo_wdg_case\r\n");

#if WDG_CASE_RST
    demo_wdg_case_rst_mode();
#elif WDG_CASE_INTR
    demo_wdg_case_int_mode();
#endif
}

