/*******************************************************************************
*
*
* File:          hal_pwm.h
* Description： pwm HAL层头文件
* Version：      V0.1
* Date：         2021-03-17
* Author：       jaya
 *******************************************************************************/
#ifndef __HAL_PWM_H__
#define __HAL_PWM_H__

/*******************************************************************************
* 1.Included files
*******************************************************************************/
#include "tau_device_datatype.h"
#include "tau_common.h"

/*******************************************************************************
* 2.Global constant and macro definitions using #define
*******************************************************************************/

/*******************************************************************************
* 3.Global structures, unions and enumerations using typedef
*******************************************************************************/

/*******************************************************************************
* 4.Global variable extern declarations
*******************************************************************************/

/*******************************************************************************
* 5.Global function prototypes
*******************************************************************************/
/**
* @brief  pwm 初始化
* @param  frequency: PWM 频率 < 30000 (30K)
* @param  duty_step: 占空比调试阶数,与硬件相关,不能超过硬件限制（建议配置255）
          最大阶数 = (1000000000/130)/frequency，常见如下
           frequency, max step(最大step可配置)
          { 30000,      255 }
          { 4000,       1923}
          { 3000,       2564}
* @retval true/false
*/
bool hal_pwm_init(uint32_t frequency, uint32_t duty_step);

/**
* @brief  pwm 启动停止控制
* @param  enable: 启动/停止
* @retval none
*/
void hal_pwm_enable(bool enable);

/**
* @brief  pwm 设置占空比
* @param  duty_ratio: 占空比,范围为0 - duty_step(hal_pwm_init配置的参数)
* @retval none
*/
void hal_pwm_set_duty(uint32_t duty_ratio);

/**
* @brief  pwm 配置从elvcc直接输出背光电源
* @param  enable: ELVCC输出启停
* @retval none
*/
bool hal_pwm_set_elvcc_output(bool enable);

/**
* @brief 配置elvcc PWM 驱动背光能力,范围 0 - 15, capactiy为15时驱动能力最强,ELVCC输出电流最大
* @param capacity: 0 - 15,hal_pwm_set_elvcc_output配置时会恢复默认值15
* @retval none
*/
void hal_pwm_set_elvcc_capacity(uint8_t capacity);

/**
* @brief  pwm 去初始化
* @param  none
* @retval true/false
*/
bool hal_pwm_deinit(void);

#endif /* __HAL_PWM_H__ */
