/*******************************************************************************
* Copyright (C) 2021-2025, All Rights Reserved.
*
* File:        hal_spi_master.h
* Description  hal spi masrer模块头文件
* Version      V0.1
* Date         2023-07-27
* Author       swx
*******************************************************************************/
#ifndef __HAL_SPI_MASTER_H__
#define __HAL_SPI_MASTER_H__

/*******************************************************************************
* 1.Included files
*******************************************************************************/
#include "tau_device_datatype.h"
#include "tau_common.h"
#include "string.h"

/*******************************************************************************
* 2.Global constant and macro definitions using #define
*******************************************************************************/

/*******************************************************************************
* 3.Global structures, unions and enumerations using typedef
*******************************************************************************/

/*******************************************************************************
* 4.Global variable extern declarations
*******************************************************************************/

/*******************************************************************************
* 5.Global function prototypes
*******************************************************************************/

/**
* @brief  初始化spi master 模块
* @param  speed: 传输速度
* @param  cpha:  相位配置
* @param  cpol:  极性配置
* @retval none
*/
void hal_spim_init(uint32_t speed, uint8_t cpha, uint8_t cpol);

/**
* @brief  spi master 去初始化
* @param  none
* @retval none
*/
void hal_spim_deinit(void);

/**
* @brief  获取dma tx 传输状态
* @param  none
* @retval true:传输完成, false:传输数据中,不可操作spim接口
*/
bool hal_spim_get_transfer_complete(void);

/**
* @brief  spim flush fifo
* @param  none
* @retval none
*/
void hal_spim_flush(void);

/**
* @brief  spim 发送数据,函数返回即传输完成,阻塞函数
* @param  tx_buffer:tx buffer地址
* @param  tx_size:  tx size
* @retval none
*/
void hal_spim_write(const uint8_t *data_buffer, uint32_t data_size);

/**
* @brief  spi master读写数据,函数返回即传输完成,阻塞函数
* @param  cmd :     传输过程汇总需要发送command的地址
* @param  cmd_size: 传输过程汇总需要发送command的数据长度
* @param  rx_buffer:rx buffer地址
* @param  rx_size:  rx buffer size
* @retval none
*/
void hal_spim_read(const uint8_t *cmd, uint32_t cmd_size, uint8_t *data_buffer, uint32_t data_size);


#endif

