/*******************************************************************************
*
*
* File:        hal_system.h
* Description  hal 通用系统接口头文件
* Version      V0.1
* Date         2021-05-21
* Author       lzy
 *******************************************************************************/
#ifndef __HAL_SYSTEM_H__
#define __HAL_SYSTEM_H__
/*******************************************************************************
* 1.Included files
*******************************************************************************/
#include "tau_common.h"
#include "hal_gpio.h"
#include "tau_log.h"

/*******************************************************************************
* 2.Global constant and macro definitions using #define
*******************************************************************************/

/*******************************************************************************
* 3.Global structures, unions and enumerations using typedef
*******************************************************************************/
/**
* @brief 系统时钟配置
*/
typedef enum
{
    HAL_SYSCLK_80M  = 80000000,
    HAL_SYSCLK_100M = 100000000,
    HAL_SYSCLK_150M = 150000000
} hal_system_clk_e;

/*******************************************************************************
* 4.Global variable extern declarations
*******************************************************************************/

/*******************************************************************************
* 5.Global function prototypes
*******************************************************************************/

/**
* @brief  system 初始化
* @param  sysclk:系统时钟
* @retval none
*/
void hal_system_init(hal_system_clk_e sysclk);

/**
* @brief  mcu进入idle模式,等待中断唤醒
* @param  disable_systick: 进入idle时是否关闭systick(退出idle 恢复systick)
* @retval none
*/
void hal_system_idle_mode(bool disable_systick);

/**
* @brief  注册systick回调函数
* @param  cb_func：回调函数地址
* @retval 无
*/
void hal_system_register_systick_cb(fcb_type cb_func);

/**
* @brief  启动sys tickt
* @param  ms: sys tickt 间隔, 范围1-10ms
* @retval true/false
*/
bool hal_system_enable_systick(uint8_t ms);

/**
* @brief  获取systickt
* @param  none
* @retval 当前systickt值
*/
bool hal_system_disable_systick(void);

/**
* @brief  获取systickt
* @param  none
* @retval 当前systickt值
*/
uint32_t hal_system_get_tick(void);

/**
* @brief  reset chip
* @param  none
* @retval none
*/
void hal_system_reset_chip(void);

/**
* @brief  获取上位机设置的debug state(debug only)
* @param  none
* @retval debug state
*/
uint32_t hal_system_get_debug_state(void);

/**
* @brief  clear debug state(debug only)
* @param  none
* @retval none
*/
void hal_system_clear_debug_state(void);

/**
* @brief 更新MCU时钟
* @param  sysclk:系统时钟
* @retval true/false
*/
bool hal_system_updata_sysclk(hal_system_clk_e sysclk);

#endif //__HAL_SYSTEM_H__
