/*******************************************************************************
*
*
* File:          hal_timer.h
* Description： timer HAL层头文件
* Version：      V0.1
* Date：         2021-03-16
* Author：       jaya
 *******************************************************************************/
#ifndef __HAL_TIMER_H__
#define __HAL_TIMER_H__

/*******************************************************************************
* 1.Included files
*******************************************************************************/
#include "tau_device_datatype.h"
#include "tau_common.h"

/*******************************************************************************
* 2.Global constant and macro definitions using #define
*******************************************************************************/

/*******************************************************************************
* 3.Global structures, unions and enumerations using typedef
*******************************************************************************/

/*******************************************************************************
* 4.Global variable extern declarations
*******************************************************************************/

/*******************************************************************************
* 5.Global function prototypes
*******************************************************************************/
/**
* @brief  指定定时器初始化
* @param  index：实例序号(0~3)，参考枚举类型timer_num_e
* @retval None
*/
void hal_timer_init(timer_num_e index);

/**
* @brief  指定定时器反初始化
* @param  index：实例序号(0~3)，参考枚举类型timer_num_e
* @retval None
*/
void hal_timer_deinit(timer_num_e index);

/**
* @brief  启动指定定时器
* @param  index：实例序号(0~3)，参考枚举类型timer_num_e
* @param  ms：超时时间，单位ms。由于应用场景一般是ms级别的，应用开发不需要计数具体时针数，
              故直接输入时间，在接口内部换算成时钟数进行寄存器设置。
* @param  cb_func：回调函数地址，不需要则填NULL
* @param  data：回调函数的参数地址，不需要则填NULL
* @retval None
*/
void hal_timer_start(timer_num_e index, uint32_t ms, fcb_type cb_func, void *data);

/**
* @brief  启动指定定时器
* @param  index：实例序号(0~3)，参考枚举类型timer_num_e
* @param  us：超时时间，单位us。由于应用场景一般是us级别的，应用开发不需要计数具体时针数，
              故直接输入时间，在接口内部换算成时钟数进行寄存器设置。
* @param  cb_func：回调函数地址，不需要则填NULL
* @param  data：回调函数的参数地址，不需要则填NULL
* @retval None
*/
void hal_timer_start_ex(timer_num_e index, uint32_t us, fcb_type cb_func, void *data);

/**
* @brief  停止指定定时器
* @param  index：实例序号(0~3)，参考枚举类型timer_num_e
* @retval None
*/
void hal_timer_stop(timer_num_e index);

/**
* @brief  设置定时器是否循环超时
* @param  index：实例序号(0~3)，参考枚举类型timer_num_e
* @param  bool enable：循环超时使能
* @retval None
*/
void hal_timer_set_repeat(timer_num_e index, bool repeat);

/**
* @brief  获取指定指示器状态
* @param  index：实例序号(0~3)，参考枚举类型timer_num_e
* @retval 参考timer_status_e
*/
timer_status_e hal_timer_get_status(timer_num_e index);

#endif /* __HAL_TIMER_H__ */
