/*******************************************************************************
*
*
* File:          hal_wdg.h
* Description： wdg HAL层头文件
* Version：      V0.1
* Date：         2021-03-16
* Author：       jaya
 *******************************************************************************/
#ifndef __HAL_WDG_H__
#define __HAL_WDG_H__

/*******************************************************************************
* 1.Included files
*******************************************************************************/
#include "tau_device_datatype.h"
#include "tau_common.h"


/*******************************************************************************
* 2.Global constant and macro definitions using #define
*******************************************************************************/

/*******************************************************************************
* 3.Global structures, unions and enumerations using typedef
*******************************************************************************/
/*!
 * @brief watch dog模式
 */
typedef enum
{
    WDG_MODE_RESET     = 0,  //复位模式，跑飞复位
    WDG_MODE_INTERRUPT = 1   //中断模式，跑飞进入中断
} wdg_mode_e;

/*******************************************************************************
* 4.Global variable extern declarations
*******************************************************************************/

/*******************************************************************************
* 5.Global function prototypes
*******************************************************************************/
/**
* @brief  看门狗初始化
* @param  无
* @retval 无
*/
void hal_wdg_init(void);

/**
* @brief  看门狗反初始化
* @param  无
* @retval 无
*/
void hal_wdg_deinit(void);

/**
* @brief  启动看门狗
* @param  wdg_mode_e modeSel: 复位或中断模式
* @param  uint32_t load: 超时时间，单位ms
* @retval 无
*/
void hal_wdg_start(wdg_mode_e modeSel, uint32_t load);

/**
* @brief  停止看门狗
* @param  无
* @retval 无
*/
void hal_wdg_stop(void);

/**
* @brief  设置WDG是否循环超时
* @param  enable：循环超时使能
* @retval 无
*/
void hal_wdg_set_repeat(bool repeat);

/**
* @brief  注册中断回调函数
* @param  cb_func：回调函数地址
* @param  data：回调参数地址
* @retval 无
*/
void hal_wdg_register_callback(fcb_type cb_func, void *data);

/**
* @brief  喂狗
* @param  无
* @retval 无
*/
void hal_wdg_kick_dog(void);

#endif /* __HAL_WDG_H__ */
